function printTable ( t )  
    local print_r_cache={}
    local function sub_print_r(t,indent)
        if (print_r_cache[tostring(t)]) then
            print(indent.."*"..tostring(t))
        else
            print_r_cache[tostring(t)]=true
            if (type(t)=="table") then
                for pos,val in pairs(t) do
                    if (type(val)=="table") then
                        print(indent.."["..pos.."] => "..tostring(t).." {")
                        sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                        print(indent..string.rep(" ",string.len(pos)+6).."}")
                    elseif (type(val)=="string") then
                        print(indent.."["..pos..'] => "'..val..'"')
                    else
                        print(indent.."["..pos.."] => "..tostring(val))
                    end
                end
            else
                print(indent..tostring(t))
            end
        end
    end
    if (type(t)=="table") then
        print(tostring(t).." {")
        sub_print_r(t,"  ")
        print("}")
    else
        sub_print_r(t,"  ")
    end
    print()
end

function dumpValue(st)
	DevTools_DumpCommand(st);
end

ItemSlots = 
{
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"WristSlot",

	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	
	"Finger0Slot",
	"Finger1Slot",
	
	"Trinket0Slot",
	"Trinket1Slot",
	
	"MainHandSlot",
	"SecondaryHandSlot"
};

function GetItemIDFromLink(link)
	return tonumber(strmatch(link, "item:(%d+):"));
end

function GetItemInfoTable(id)
	local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(id);

	local itemInfo = 
	{
		name = name,
		link = link,
		quality = quality,
		iLevel = iLevel,
		reqLevel = reqLevel,
		class = class,
		subclass = subclass,
		maxStack = maxStack,
		equipSlot = equipSlot,
		texture = texture,
		vendorPrice = vendorPrice
	};

	return itemInfo;

end

function GetCurrentGearLinksForAPlayer(player)
	local gear = {};

	for i,itemSlot in pairs(ItemSlots) do

		local slot = GetInventorySlotInfo(itemSlot);
		local link = GetInventoryItemLink(player, slot);

		if(link)then
			gear[itemSlot] = link;
		else
			gear[itemSlot] = nil;
		end
	end

	return gear;

end

function GetCurrentGearIDsForAPlayer()
	local gear = {};

	for i,itemSlot in pairs(ItemSlots) do

		local slot = GetInventorySlotInfo(itemSlot);
		local id = GetInventoryItemID("player", slot);

		if(id)then
			gear[itemSlot] = id;
		else
			gear[itemSlot] = nil;
		end
	end

	return gear;
end

function RakijaAddon_Round(num, idp)
	return tonumber(string.format("%." .. (idp or 0) .. "f", num))
end

function  GetCurrentPlayerFullName(who)
	local fullName;
	local name,realm = UnitFullName(who);
	if(realm and realm ~= "")then
		fullName = name.."-"..realm;
	else
		fullName = name.."-"..string.gsub(GetRealmName(),"%s+","");
	end

	return name,fullName;
end

function UnpackProfData(ProfData)
	return ProfData.name,ProfData.texture,ProfData.rank,ProfData.maxStack,ProfData.numSpells,ProfData.spelloffset,ProfData.skillLine,ProfData.rankModifier,ProfData.specializationIndex,ProfData.specializationOffset;
end

function GetSkillIDForProf(texture)
	for i=1,GetNumGuildTradeSkill() do
		local skillID, _, iconTexture = GetGuildTradeSkillInfo(i);
		if(iconTexture == texture)then
			return skillID;
		end
	end
end

function __genOrderedIndex( t )
    local orderedIndex = {}
    for key in pairs(t) do
        table.insert( orderedIndex, key )
    end
    table.sort( orderedIndex )
    return orderedIndex
end

function orderedNext(t, state)
    -- Equivalent of the next function, but returns the keys in the alphabetic
    -- order. We use a temporary ordered key table that is stored in the
    -- table being iterated.

    local key = nil
    --print("orderedNext: state = "..tostring(state) )
    if state == nil then
        -- the first time, generate the index
        t.__orderedIndex = __genOrderedIndex( t )
        key = t.__orderedIndex[1]
    else
        -- fetch the next value
        for i = 1,table.getn(t.__orderedIndex) do
            if t.__orderedIndex[i] == state then
                key = t.__orderedIndex[i+1]
            end
        end
    end

    if key then
        return key, t[key]
    end

    -- no more value to return, cleanup
    t.__orderedIndex = nil
    return
end

function orderedPairs(t)
    -- Equivalent of the pairs() function on tables. Allows to iterate
    -- in order
    return orderedNext, t, nil
end

RAKIJA_FOOD_BUFFS = 
{
	[225603]=375,
	[225604]=375,
	[225602]=375,
	[225605]=375,
	[225606]=375
};

RAKIJA_FLASK_BUFFS = 
{
	[188035]=1300,
	[188031]=1300,
	[188033]=1300,
	[188034]=1300
};

RAKIJA_VANTUS_BUFFS = 
{
	[192761] = 1000,
	[192762] = 1000,
	[192763] = 1000,
	[192764] = 1000,
	[192765] = 1000,
	[192766] = 1000,
	[192767] = 1000,
	[229174] = 1000,
	[229175] = 1000,
	[229176] = 1000
};

RAKIJA_RUNE_BUFF = 224001;

function RakijaAddon_GetUnitBuffs(groupDescriptor)
	local buffs = {};
	local name = "stub";

	local iterator = 1;

	local name, rank, icon, count, _, duration, expires,_, _, _, spellID;

	while true do

		name, rank, icon, count, _, duration, expires,_, _, _, spellID= UnitAura(groupDescriptor,iterator,"HELPFUL");
		if(not name)then
			break;
		end

		buffs[spellID] = {};
		buffs[spellID].name = name;
		buffs[spellID].rank = rank;
		buffs[spellID].icon = icon;
		buffs[spellID].count = count;
		buffs[spellID].duration = duration;
		buffs[spellID].expires = expires;
		buffs[spellID].spellID = spellID;

		iterator= iterator +1;

	end

	return buffs;
end

function RakijaAddon_GetGroupDiscriptor(index)

	local groupDescriptor;

	if(IsInRaid())then
		groupDescriptor = "raid"..index;
	else
		if(index == 1)then
			groupDescriptor = "player";
		else
			groupDescriptor = "party"..index - 1;
		end
	end

	return groupDescriptor;
end

function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end


local function SortRaidersFnc(t,a,b)
	local itemIndex = RakijaLootList.selectedButton;
	if(not t[a].itemRolls[itemIndex])then
		return false;
	elseif(not t[b].itemRolls[itemIndex])then
		return true;
	end

	if(t[a].itemRolls[itemIndex].rolled == 4)then
		return false;
	end

	if(t[b].itemRolls[itemIndex].rolled == 4)then
		return true;
	end

	if(t[a].itemRolls[itemIndex].itemLvlDiff2 and not t[b].itemRolls[itemIndex].itemLvlDiff2)then
		return true;
	end

	if(not t[a].itemRolls[itemIndex].itemLvlDiff2 and t[b].itemRolls[itemIndex].itemLvlDiff2)then
		return false;
	end

	if(t[a].itemRolls[itemIndex].itemLvlDiff1 and not t[b].itemRolls[itemIndex].itemLvlDiff1)then
		return true;
	end

	if(not t[a].itemRolls[itemIndex].itemLvlDiff1 and t[b].itemRolls[itemIndex].itemLvlDiff1)then
		return false;
	end

	if(not t[a].itemRolls[itemIndex].itemLvlDiff1 and not t[b].itemRolls[itemIndex].itemLvlDiff1)then
		return false;
	end

	if(t[a].itemRolls[itemIndex].itemLvlDiff2 and t[b].itemRolls[itemIndex].itemLvlDiff2)then
		local player1diff,player2diff;
		if(t[a].itemRolls[itemIndex].itemLvlDiff2 > t[a].itemRolls[itemIndex].itemLvlDiff1)then
			player1diff = t[a].itemRolls[itemIndex].itemLvlDiff2;
		else
			player1diff = t[a].itemRolls[itemIndex].itemLvlDiff1;
		end

		if(t[b].itemRolls[itemIndex].itemLvlDiff2 > t[b].itemRolls[itemIndex].itemLvlDiff1)then
			player2diff = t[b].itemRolls[itemIndex].itemLvlDiff2;
		else
			player2diff = t[b].itemRolls[itemIndex].itemLvlDiff1;
		end

		return player1diff > player2diff;
	end

	--print(t[b].itemRolls[itemIndex].itemLvlDiff1 > t[a].itemRolls[itemIndex].itemLvlDiff1);
	return t[b].itemRolls[itemIndex].itemLvlDiff1 < t[a].itemRolls[itemIndex].itemLvlDiff1;
end	

function RakijaAddon_GetSortedRaidersByIlvl()
	local table = RakijaDBLootTable.raiders;
	local raiders = {};
	local index = 1;
	for k,v in spairs(table,SortRaidersFnc) do
		local temp = v;
		temp.name = k;
		raiders[index] = temp;
		index = index + 1;
	end

	return raiders;
end
